# Generated by Django 4.2.23 on 2025-06-20 09:52

# Django
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("aadiscordbot", "0016_authbotconfiguration_admin_user_ids_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="Setting",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "applicant_role_name",
                    models.CharField(
                        default="",
                        help_text="The name of the role for applicants in the Discord server.",
                        max_length=100,
                        verbose_name="Applicant Role Name",
                    ),
                ),
                (
                    "applicant_role_id",
                    models.PositiveBigIntegerField(
                        default=None,
                        help_text="The ID of the role for applicants in the Discord server.",
                        null=True,
                        verbose_name="Applicant Role ID",
                    ),
                ),
                (
                    "recruiter_role_id",
                    models.PositiveBigIntegerField(
                        default=None,
                        help_text="The ID of the role for recruiters in the Discord server.",
                        null=True,
                        verbose_name="Recruiter Role ID",
                    ),
                ),
                (
                    "leadership_role_id",
                    models.PositiveBigIntegerField(
                        default=None,
                        help_text="The ID of the role for leadership in the Discord server.",
                        null=True,
                        verbose_name="Leadership Role ID",
                    ),
                ),
                (
                    "welcome_roles_excluded",
                    models.CharField(
                        default="Member",
                        help_text="Comma-separated list of Discord roles that are excluded from the welcome message.",
                        max_length=255,
                        verbose_name="Roles Excluded",
                    ),
                ),
                (
                    "recruiting_channel",
                    models.ForeignKey(
                        help_text="The channel used for recruiting in the Discord server.",
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to="aadiscordbot.channels",
                        verbose_name="Recruiting Channel",
                    ),
                ),
                (
                    "welcome_channel_authenticated",
                    models.ForeignKey(
                        help_text="The channel to send welcome messages for authenticated users in the Discord server.",
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to="aadiscordbot.channels",
                        verbose_name="Welcome Channel (Authenticated User)",
                    ),
                ),
                (
                    "welcome_channel_unauthenticated",
                    models.ForeignKey(
                        help_text="The channel to send welcome messages for unauthenticated users in the Discord server.",
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to="aadiscordbot.channels",
                        verbose_name="Welcome Channel (Unauthenticated User)",
                    ),
                ),
            ],
            options={
                "verbose_name": "Setting",
                "verbose_name_plural": "Settings",
                "default_permissions": (),
            },
        ),
    ]
