# Generated by Django 4.2.23 on 2025-07-13 10:14

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("tnnt_discordbot_cogs", "0003_setting_locate_channels"),
    ]

    operations = [
        migrations.CreateModel(
            name="Permission",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "verbose_name": "Command Permission",
                "permissions": (
                    ("locate", "Can run the `/locate` command"),
                    ("lookup", "Can run the `/lookup` command"),
                ),
                "managed": False,
                "default_permissions": (),
            },
        ),
    ]
