# Generated by Django 4.2.23 on 2025-07-10 15:34

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("aadiscordbot", "0016_authbotconfiguration_admin_user_ids_and_more"),
        ("tnnt_discordbot_cogs", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="setting",
            name="lookup_channels",
            field=models.ManyToManyField(
                blank=True,
                help_text="Channels in which the `/lookup` command can be used.",
                related_name="lookup_channels",
                to="aadiscordbot.channels",
                verbose_name="Lookup Channels",
            ),
        ),
    ]
